package mi.hdm.controllers;

import javafx.fxml.FXML;
import javafx.scene.Node;
import javafx.scene.control.CheckBox;
import javafx.scene.control.Label;
import javafx.scene.layout.AnchorPane;
import javafx.scene.layout.HBox;
import javafx.scene.layout.TilePane;
import javafx.scene.layout.VBox;
import mi.hdm.recipes.*;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class MainPageController extends BaseController {
    private final static Logger log = LogManager.getLogger(MainPageController.class);

    private final RecipeManager recipeManager;
    private final CategoryManager categoryManager;

    private Map<Category, CheckBox> categoryToCheckbox = new HashMap<>();
    private final List<VBox> currentRecipesInGUI = new ArrayList<>(); //keeps track of all Nodes in the GUI that represent a recipe
    private List<Category> currentlySelectedCategories = new ArrayList<>();

    @FXML
    private AnchorPane parent;
    @FXML
    private TilePane recipeTilePane;
    @FXML
    private HBox categories;

    public MainPageController(RecipeManager recipeManager, CategoryManager categoryManager) {
        this.recipeManager = recipeManager;
        this.categoryManager = categoryManager;
    }

    @FXML
    public void initialize() {
        loadHeader(parent);
        mapRecipes(recipeManager.getRecipes());
        mapCategories();
        log.debug("Added {} recipes to GUI", recipeManager.getRecipes().size());
    }

    @FXML
    public void addRecipe() {
        log.info("User is trying to add recipe!");
    }

    private void mapRecipes(List<Recipe> recipes) {
        //TODO: every recipe tile needs an event listener
        recipeTilePane.getChildren().removeAll(currentRecipesInGUI);
        currentRecipesInGUI.clear();
        for (Recipe recipe : recipes) {
            Label recipeName = new Label(recipe.getName());
            Label recipeDescription = new Label(recipe.getDescription());
            List<Label> categoryLabels = new ArrayList<>();

            //add a maximum of 3 categories to the recipe tile
            for (int i = 0; i < Math.min(recipe.getCategories().size(), 3); i++) {
                Category c = recipe.getCategories().get(i);
                Label l = new Label(c.getName());
                //TODO: make border radius work on the styling
                l.setStyle("-fx-background-color: " + c.getColourCode() + ";" +
                        "-fx-padding: 3px;" +
                        "-fx-border-radius: 5px;"
                );
                categoryLabels.add(l);
            }
            HBox categories = new HBox();
            categories.getChildren().addAll(categoryLabels);

            VBox currentRecipeBox = new VBox(recipeName, recipeDescription, categories);

            //TODO: make this recipe box a component class
            recipeName.setStyle(
                    "-fx-font-weight: 800;" +
                            "-fx-font-scale: 16"
            );

            currentRecipeBox.setPrefWidth(180);
            currentRecipeBox.setPrefHeight(250);
            currentRecipeBox.setStyle(
                    "-fx-background-color: #DEDEDE;" +
                            "-fx-border-color: #D91C1C;" +
                            "-fx-border-radius: 8;" +
                            "-fx-border-width: 2;" +
                            "-fx-padding: 8"
            );
            currentRecipesInGUI.add(currentRecipeBox);
        }
        recipeTilePane.getChildren().addAll(currentRecipesInGUI);
    }

    private void mapCategories() {
        for (final Category category : categoryManager.getAllCategories()) {
            CheckBox categoryCheckBox = new CheckBox(category.getName());
            categoryCheckBox.setStyle("-fx-background-color:" + category.getColourCode() + ";" + "-fx-padding: 4px;");
            categoryCheckBox.setOnAction(event -> updateRecipes());

            categoryToCheckbox.put(category, categoryCheckBox);

            categories.getChildren().add(categoryCheckBox);
        }
    }

    private void updateRecipes() {
        mapRecipes(fetchSelectedRecipes());
    }

    private List<Recipe> fetchSelectedRecipes() {
        currentlySelectedCategories = categoryToCheckbox.keySet()
                .stream().filter(category ->
                        categoryToCheckbox.get(category).isSelected())
                .toList();
        RecipeSearch recipeSearch = new RecipeSearch(recipeManager.getRecipes());
        return recipeSearch.searchByCategory(currentlySelectedCategories);
    }
}

class RecipeTile extends Node {

}
